﻿using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.SignalR;
using System.Threading;
using System.Reactive.Linq;
using DeviceModelLib;

namespace DashboardApp2.Hubs
{
    public class WebHub : Hub
    {
        CancellationTokenSource cts = new CancellationTokenSource();

        public WebHub()
        {
            InterHubHelper.Instance.AddWebHub(this);
        }

        public InitInfoResult GetInitInfo()
        {
            return new InitInfoResult
                {
                    DeviceTypes = InterHubHelper.Instance.GetDeviceTypes(),
                    Groups = InterHubHelper.Instance.GetGroups(),
                    Devices = InterHubHelper.Instance.GetDevices(),

                    //TEST
                    //Devices = InterHubHelper.Instance.GetDevices(),

                    //DeviceTypes = new DeviceTypeInfo[]
                    //    {
                    //        new DeviceTypeInfo { Id = 1, Name = "Temperature", Units = "degree C", Min = -10, Max = 110, Description = "Ga!" },
                    //    },

                    //Devices = new DeviceInfo[]
                    //    {
                    //         new DeviceInfo { Id = 19, Name = "My Device", TypeId = 1, Description = "ABC" },
                    //    },

                    //Groups = new GroupInfo[]
                    //    {
                    //         new GroupInfo { Id = 87, Name = "The Group", Description = "This is the Group" },
                    //    }
                };
        }

        public void RegisterDeviceWithWeb(DeviceInfo deviceInfo)
        {
            Clients.All.SendAsync("registerDeviceWithWeb", new DevicesResult { Device = deviceInfo });
        }

        public IObservable<MeasurementsResult> StartStreaming()
        {
            return Observable.Create<MeasurementsResult>(async observer =>
                {
                    while (!cts.Token.IsCancellationRequested)
                    {
                        await Task.Delay(InterHubHelper.Instance.StreamingIntervalInMs, cts.Token);
                        observer.OnNext(new MeasurementsResult { Measurements = InterHubHelper.Instance.MeasurementsObj });
                    }
                });
        }

        public void CancelStreaming()
        {
            cts.Cancel();
        }

        protected override void Dispose(bool disposing)
        {
            InterHubHelper.Instance.RemoveWebHub(this);
        }
    }

    public class InitInfoResult
    {
        public DeviceInfo[] Devices { get; set; }
        public DeviceTypeInfo[] DeviceTypes { get; set; }
        public GroupInfo[] Groups { get; set; }
    }

    public class DevicesResult
    {
        public DeviceInfo Device { get; set; }
    }

    public class MeasurementsResult
    {
        public Measurement[] Measurements { get; set; }
    }
}
