﻿using System;
using System.Threading;
using System.Threading.Tasks;
using System.Text;

namespace DevicesSimulator
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.Write("Devices Simulator\nPress any key to start generate measurements...\n");
            Console.ReadKey();

            HttpHelper httpHelper = new HttpHelper { User = "...", Password = "..." };

            httpHelper.RegisterDevice(/* deviceId */           101,
                                      /* typeId */               1,
                                      /* deviceName */         "T1-1",
                                      /* description */        "Temperature in Box 1-1, ℃",
                                      /* deviceSpecificData */ "ABC");
            int i = 0;

            httpHelper.RegisterDevice(201, 1, "T2-1", "Temperature in Box 2-1, ℃", "ABC");
            httpHelper.RegisterDevice(202, 1, "T2-2", "Temperature in Box 2-2, ℃", "ABC");
            httpHelper.RegisterDevice(102, 1, "T1-2", "Temperature in Box 1-2, ℃", "ABC");
            httpHelper.RegisterDevice(203, 1, "T2-3", "Temperature in Box 2-3, ℃", "ABC");
            httpHelper.RegisterDevice(204, 1, "T2-4", "Temperature in Box 2-4, ℃", "ABC");

            int intervalInMs = 3000;

            Thread.Sleep(intervalInMs);

            while (true)
            {
                Task.Run(() =>
                    {
                        httpHelper.SendMeasurement(101, Calc(i, 0) * 0.3 + 30);
                        httpHelper.SendMeasurement(102, Calc(i, 2) * 0.3 + 30);
                        httpHelper.SendMeasurement(201, Calc(i, 4) * 0.3 + 30);
                        httpHelper.SendMeasurement(202, Calc(i, 6) * 0.2 + 15);
                        httpHelper.SendMeasurement(203, Calc(i, 8));

                        if (i == 5)
                            httpHelper.SendMeasurement(204, 10 * i);
                    });

                Thread.Sleep(intervalInMs);
                i = ++i % 10;
            }
        }

        static float Calc(int i, int offset)
        {
            return (float)((Math.Sin((i + offset) * Math.PI / 10) + 1) * 50);
        }
    }

    public static class HttpHelperEx
    {
        private static string UriHeader
        {
            get { return "http://localhost:57523/api/device/"; }
        }

        public static void RegisterDevice(this HttpHelper httpHelper, params object[] items)
        {
            Get(httpHelper, UriHeader + string.Format("RegisterDevice/{0}/{1}/{2}/{3}/{4}", items));
        }

        public static void SendMeasurement(this HttpHelper httpHelper, params object[] items)
        {
            Get(httpHelper, UriHeader + string.Format("SendMeasurement/{0}/{1}", items));
        }

        public static void Get(this HttpHelper httpHelper, string url)
        {
            Task.Run(() =>
                {
                    const int iTimeoutInSec = 60;
                    var btOut = httpHelper.Get(url, iTimeoutInSec);
                    var sr = Encoding.UTF8.GetString(btOut, 0, btOut.Length);
                    Console.WriteLine(sr);
                });
        }
    }
}
