import { Component } from '@angular/core';
import { CommunicationService, DeviceMeasurement } from '../providers/communication.service';
import { Chart } from 'chart.js';
import { Subscription } from 'rxjs/Subscription';

@Component({
    template: '',
})
export class ChartComponent {

    chart: Chart;
    private values = [];
    private min;
    static mapChart: Map<number, Chart> = new Map<number, Chart>();
    subscription: Subscription;
         
    constructor(private deviceId: number) {
        var measurements = CommunicationService.comm.getMeasurementsByDeviceId(deviceId);
        if (measurements && measurements.length > 0)
            for (var i = 0; i < measurements.length; i++)
                this.values.push(measurements[i].value);

        CommunicationService.comm.receiveMeasurementSubject.asObservable().subscribe(() => this.updateChart());
    }

    updateChart() {  
        var measurement: DeviceMeasurement = CommunicationService.comm.getLastMeasurementByDeviceId(this.deviceId);
        if (!measurement)
            return;

        this.values.push(measurement.value == 'NaN' ? this.min * 5 : measurement.value);
        if (this.values.length > CommunicationService.comm.ARR_MAX_LENGTH)
            this.values.shift();

        
        if (this.chart) {
            this.setChartColor(measurement.status);
            this.chart.update();
        }
    }

    setChartColor(status) {
        var borderColor;
        var backgroundColor;

        switch (status) {
            case 1:
                borderColor =     'rgb(0, 128, 0)';
                backgroundColor = 'rgba(0, 128, 0, 0.2)';
                break;

            case 2:
                borderColor =     'rgb(255, 128, 0)';
                backgroundColor = 'rgba(255, 128, 0, 0.2)';
                break;

            case 3:
                borderColor =     'rgb(255, 0, 0)';
                backgroundColor = 'rgba(255, 0, 0, 0.2)';
                break;
        }

        var dataset = this.chart.data.datasets[0];
        dataset.borderColor = borderColor;
        dataset.backgroundColor = backgroundColor;
    }

    createChart(min, max, units) {
        var htmlElement = document.getElementById('chart' + this.deviceId);
        if (!htmlElement) 
            return;       

        this.min = min;
        
        var labels = [];
        for (var i = -(CommunicationService.comm.ARR_MAX_LENGTH - 1); i <= 0; i++)
            labels.push(-i);

        if (!ChartComponent.mapChart.get(this.deviceId))
        {
            var chart = new Chart(htmlElement, 
                {
                    type: 'line',
                    data: {
                        datasets: [
                            {
                                data: this.values,
                                borderWidth: 3,
                                lineTension: 0.25,
                                pointRadius: 0,
                            }
                        ]
                    },
                    
                    options: {
                        animation: {
                            duration: 0,
                        },
                        hover: {
                            animationDuration: 0,
                        },
                        responsiveAnimationDuration: 0,
                        legend: false,
                        scales: {
                            xAxes: [{
                                type: 'category',
                                labels: labels,
                            }],
                            yAxes: [
                                {
                                    display: true,
                                    ticks: {
                                        min: min,
                                        max: max
                                    },
                                    scaleLabel: {
                                        display: true,
                                        labelString: units,
                                        fontStyle: 'italic',
                                        lineHeight: 0.8,
                                        padding: 1
                                    }
                                }
                            ]
                        }
                    }                    
                }); 

            ChartComponent.mapChart.set(this.deviceId, chart);
        }

        this.chart = ChartComponent.mapChart.get(this.deviceId);
    }
}